"""Process a TCX or GPX file into a dataframe."""
from pathlib import Path
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

import numpy as np
import pandas as pd
from defusedxml.ElementTree import parse


class TrackingColumn:
    """Names of columns in a tracking dataframe."""

    TIME = "time"
    LATITUDE = "lat"
    LONGITUDE = "lon"
    ALTITUDE = "alt"
    HEARTBEAT = "bpm"


def _float(x: Optional[str]) -> float:
    if x is not None:
        return float(x)
    else:
        return np.nan


def _data_to_dataframe(data: List[Dict[str, Any]]) -> pd.DataFrame:
    df = pd.DataFrame(data)
    df[TrackingColumn.TIME] = pd.to_datetime(df[TrackingColumn.TIME])
    return df


def tcx_to_dataframe(tcx: Path) -> pd.DataFrame:
    """Process a TCX file."""
    tree = parse(str(tcx))
    ns = {"tcx": "http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2"}

    trackpoints = tree.findall(".//tcx:Trackpoint", ns)
    n_trackpoints = len(trackpoints)
    print(f"Tracking {n_trackpoints} trackpoints")
    data = []
    for point in trackpoints:
        sample = {
            TrackingColumn.TIME: point.find(".//tcx:Time", ns).text,
            TrackingColumn.LATITUDE: _float(
                point.find(".//tcx:LatitudeDegrees", ns).text
            ),
            TrackingColumn.LONGITUDE: _float(
                point.find(".//tcx:LongitudeDegrees", ns).text
            ),
            TrackingColumn.ALTITUDE: _float(
                point.find(".//tcx:AltitudeMeters", ns).text
            ),
            TrackingColumn.HEARTBEAT: int(
                point.find(".//tcx:HeartRateBpm/tcx:Value", ns).text
            ),
        }
        data.append(sample)

    df = _data_to_dataframe(data)
    return df


def gpx_to_dataframe(gpx: Path) -> pd.DataFrame:
    """Process GPX file."""
    tree = parse(str(gpx))
    ns = {"gpx": "http://www.topografix.com/GPX/1/1"}

    trackpoints = tree.findall(".//gpx:trkpt", ns)
    n_trackpoints = len(trackpoints)
    print(f"Tracking {n_trackpoints} trackpoints")
    data = []
    for point in trackpoints:
        sample = {
            TrackingColumn.TIME: point.find(".//gpx:time", ns).text,
            TrackingColumn.LATITUDE: _float(point.get("lat")),
            TrackingColumn.LONGITUDE: _float(point.get("lon")),
            TrackingColumn.ALTITUDE: _float(point.find(".//gpx:ele", ns).text),
        }
        data.append(sample)

    df = _data_to_dataframe(data)
    return df



"""Command-line interface."""
from pathlib import Path
from typing import Any

import click

from .heatmap import heatmap as do_heatmap
from .input_file import gpx_to_dataframe
from .input_file import tcx_to_dataframe
from .speed import plot_speed
from .speed import plot_speed_moving_avg
from .webserver import run_webserver


@click.group()
@click.version_option()
def main() -> None:
    """Visualize Tracking Data."""


@main.command()
@click.option(
    "--tcx",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
)
@click.option(
    "--to",
    type=click.Path(
        exists=False, file_okay=True, dir_okay=False, writable=True, path_type=Path
    ),
)
def tcx_to_csv(**kwargs: Any) -> None:
    """TCX file to CSV dataframe."""
    to = kwargs.pop("to")
    df = tcx_to_dataframe(**kwargs)
    df.to_csv(to, index=False)


@main.command()
@click.option(
    "--gpx",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
)
@click.option(
    "--to",
    type=click.Path(
        exists=False, file_okay=True, dir_okay=False, writable=True, path_type=Path
    ),
)
def gpx_to_csv(**kwargs: Any) -> None:
    """GPX file to CSV dataframe."""
    to = kwargs.pop("to")
    df = gpx_to_dataframe(**kwargs)
    df.to_csv(to, index=False)


@main.command()
@click.option(
    "--track",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
)
@click.option(
    "--img",
    type=click.Path(
        exists=False, file_okay=True, dir_okay=False, writable=True, path_type=Path
    ),
)
def speed(track: Path, img: Path) -> None:
    """CVS dataframe to Speed plot."""
    fig = plot_speed(track=track)
    fig.savefig(img)


@main.command()
@click.option(
    "--track",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
)
@click.option(
    "--img",
    type=click.Path(
        exists=False, file_okay=True, dir_okay=False, writable=True, path_type=Path
    ),
)
def speed_moving(track: Path, img: Path) -> None:
    """CVS dataframe to Speed plot."""
    fig = plot_speed_moving_avg(track=track)
    fig.savefig(img)


@main.command()
@click.option(
    "--track",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
)
@click.option(
    "--config",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
)
@click.option(
    "--img",
    type=click.Path(
        exists=False, file_okay=True, dir_okay=False, writable=True, path_type=Path
    ),
)
def heatmap(track: Path, config: Path, img: Path) -> None:
    """CSV dataframe to heatmap."""
    fig = do_heatmap(track=track, config=config)
    fig.savefig(img)


@main.command()
@click.option("--host", type=str, default="127.0.0.1")
@click.option("--port", type=int, default=5000)
def flask(**kwargs: Any) -> None:
    """Have a web server GUI."""
    run_webserver(**kwargs)


if __name__ == "__main__":
    main(prog_name="track-viz")  # pragma: no cover



"""Create a heatmap from the tracking information, superimposes on a background image."""
from pathlib import Path

import matplotlib as mpl
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import yaml

from .input_file import TrackingColumn


def heatmap_from_dataframe(track: pd.DataFrame, config: Path) -> mpl.figure.Figure:
    """Create heatmap."""
    with config.open("r") as src:
        conf = yaml.safe_load(src)

    img_data = plt.imread(conf["heatmap"]["background"]["image"])

    min_lon, max_lon = (
        conf["heatmap"]["background"]["min_lon"],
        conf["heatmap"]["background"]["max_lon"],
    )
    min_lat, max_lat = (
        conf["heatmap"]["background"]["min_lat"],
        conf["heatmap"]["background"]["max_lat"],
    )

    track["lon_x"] = track[TrackingColumn.LONGITUDE].apply(
        lambda x: img_data.shape[0] * (x - min_lon) / (max_lon - min_lon)
    )
    track["lat_y"] = track[TrackingColumn.LATITUDE].apply(
        lambda y: img_data.shape[1] * (y - min_lat) / (max_lat - min_lat)
    )

    fig, ax = plt.subplots(figsize=(10, 10))

    # Draw the heatmap
    sns.kdeplot(
        data=track,
        x="lon_x",
        y="lat_y",
        levels=conf["heatmap"]["plot"]["levels"],
        fill=True,
        cmap=conf["heatmap"]["plot"]["colormap"],
        ax=ax,
    )

    # Draw the satellite image
    ax.imshow(img_data, extent=[0, img_data.shape[0], 0, img_data.shape[1]])

    # Disappear ticks / labels on axes
    ax.set_ylabel("")
    ax.set_xlabel("")
    ax.set_xticks([])
    ax.set_yticks([])

    return fig


def heatmap(track: Path, config: Path) -> mpl.figure.Figure:
    """Create heatmap."""
    df = pd.read_csv(track, parse_dates=["time"])
    return heatmap_from_dataframe(track=df, config=config)



"""Create a speed graph from a tracking dataframe."""
from pathlib import Path

import matplotlib as mpl
import matplotlib.dates as mdates
import matplotlib.figure
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from geopy import distance

from .input_file import TrackingColumn

mpl.use("Agg")
sns.set_theme(
    style="whitegrid",
    palette="colorblind",
    rc={
        "axes.facecolor": "#F5FFFA",
        "figure.facecolor": "#F5FFFA",
        "figure.figsize": (20, 10),
    },
)
plt.rcParams["figure.figsize"] = (20, 10)


def track_2_movements(df: pd.DataFrame) -> pd.DataFrame:
    """Transform tracking information to time series of speed and other metrics.

    List of fields in the movements dataframe:
    * delta_time: time difference between one sample and the previous one
    * prev_lon, prev_lat: longitude / latitude of previous tracking point
    * delta_alt_m: altitude difference in meters between current sample and previous sample
    * elapsed_time: timestamp of a tracking point, considering the run started on Jan 1 2021 at 00:00
    * ground_distance_m: distance over earth surface covered between previous point and current point, in meters
    * distance_m: distance in meters between previous point and current point, taking altitude difference into account
    * speed_ms: speed at the current point in meters per second
    * speed_kmh: speed in kilometers per hour
    * speed_moving_avg_1min: moving average of the speed over the last minute, in kilometers per hour
    * delta_speed_ms: change in speed in meters per second
    * delta_moving_avg_1min: change in moving average in kilometers per hour
    * acceleration_ms2: acceleration in meters per square meters
    * use_point: bool indicates whether the point is "usable"

    The sampling frequency is identified as the most frequent time difference between 2 samples. Any time this time
    difference is more than 2 times this sampling frequency, we mark the first point AFTER the gap with FALSE in
    field "use_point".
    """
    movements = df.copy(deep=True)
    movements["delta_time"] = df[TrackingColumn.TIME].diff()
    for coord in [TrackingColumn.LONGITUDE, TrackingColumn.LATITUDE]:
        movements[f"prev_{coord}"] = movements[coord].shift()
    movements["delta_alt_m"] = movements[TrackingColumn.ALTITUDE].diff()
    movements = movements.dropna()

    # only pd.timestamp  has strftime, this is a dirty trick
    # noinspection PyTypeChecker
    movements["elapsed_time"] = (
        pd.Timestamp("2021-01-01 00:00") + movements["delta_time"].cumsum()
    )
    movements = movements.set_index("elapsed_time")

    # geopy gives only geodesic distance
    movements["ground_distance_m"] = movements.apply(
        lambda x: distance.distance(
            (x[TrackingColumn.LATITUDE], x[TrackingColumn.LONGITUDE]),
            (
                x[f"prev_{TrackingColumn.LATITUDE}"],
                x[f"prev_{TrackingColumn.LONGITUDE}"],
            ),
        ).meters,
        axis=1,
    )

    # accounting for altitude change, using Pythagorus
    movements["distance_m"] = (
        movements["ground_distance_m"] ** 2 + movements["delta_alt_m"] ** 2
    ) ** 0.5

    movements["speed_ms"] = movements["distance_m"] / movements["delta_time"].dt.seconds
    movements["speed_kmh"] = movements["speed_ms"] * 3.6
    movements["speed_moving_avg_1min"] = movements["speed_kmh"].rolling("60s").mean()

    movements["delta_speed_ms"] = movements["speed_ms"].diff()
    movements["delta_moving_avg_1min"] = movements["speed_moving_avg_1min"].diff()
    movements = movements.dropna()

    movements["acceleration_ms2"] = (
        movements["delta_speed_ms"] / movements["delta_time"].dt.seconds
    )

    # Identify missing points in the data
    freq_s = movements["delta_time"].dt.seconds.value_counts().index[0]
    movements["use_point"] = movements["delta_time"].dt.seconds.apply(
        lambda x: x <= 2 * freq_s
    )

    for measure in ["speed_moving_avg_1min", TrackingColumn.ALTITUDE]:
        movements[measure] = movements[["use_point", measure]].apply(
            lambda x: x[measure] if x["use_point"] else np.nan, axis=1
        )

    return movements


def plot_movement_field(movements: pd.DataFrame, mvt_field: str) -> mpl.figure.Figure:
    """Standard plot for 1 field of the movements dataframe."""
    fig, ax = plt.subplots()
    movements.plot.area(y=mvt_field, stacked=False, ax=ax)
    ax.xaxis.set_major_formatter(mdates.DateFormatter("%M:%S"))
    return fig


def _from_track_to_plot(track: Path, mvt_field: str) -> mpl.figure.Figure:
    df = pd.read_csv(track, parse_dates=["time"])
    movements = track_2_movements(df)
    return plot_movement_field(movements=movements, mvt_field=mvt_field)


def plot_speed(track: Path) -> mpl.figure.Figure:
    """Create the speed plot."""
    return _from_track_to_plot(track=track, mvt_field="speed_kmh")


def plot_speed_moving_avg(track: Path) -> mpl.figure.Figure:
    """Create the speed plot."""
    return _from_track_to_plot(track=track, mvt_field="speed_moving_avg_1min")


def plot_acceleration(track: Path) -> mpl.figure.Figure:
    """Create the speed plot."""
    return _from_track_to_plot(track=track, mvt_field="acceleration_ms2")


def web_plot_speed_elevation(track: pd.DataFrame) -> mpl.figure.Figure:
    """Create plot for website with both speed and elevation."""
    movs = track_2_movements(track)
    unused_points = movs.index[~movs["use_point"]]
    unused_times = []
    for pt in unused_points:
        loc = movs.index.get_loc(pt)
        unused_times.append(
            (movs.index[loc - 1], movs.index[min(movs.shape[0] - 1, loc + 1)])
        )

    fig, ax = plt.subplots()
    movs.plot.line(
        y=["speed_moving_avg_1min", "alt"],
        secondary_y=["alt"],
        ax=ax,
        linewidth=2,
        color=[sns.color_palette()[0], sns.color_palette()[1]],
        zorder=0,
    )
    movs.plot.area(
        y=["speed_moving_avg_1min", "alt"],
        secondary_y=["alt"],
        stacked=False,
        ax=ax,
        linewidth=0,
        color=[sns.color_palette()[0], sns.color_palette()[1]],
        alpha=0.2,
        legend=False,
        zorder=1,
    )

    for u in unused_times:
        ax.fill_between(
            [u[0], u[1]],
            ax.get_ylim()[0],
            ax.get_ylim()[1],
            facecolor="lightgrey",
            hatch="//",
            zorder=2,
        )

    ax.grid(visible=False)
    ax.xaxis.set_major_formatter(mdates.DateFormatter("%M:%S"))

    # Elevation graph within the bottom third of the image
    ax.right_ax.set_ylim(
        movs["alt"].min() - 0.5,
        movs["alt"].max() + 2 * (movs["alt"].max() - movs["alt"].min()),
    )
    ax.right_ax.grid(visible=False)
    return fig


def web_plot_speed_climb_kde(track: pd.DataFrame) -> mpl.figure.Figure:
    """Create plot for website for KDE with speed and elevation."""
    movs = track_2_movements(track)
    movs = movs.replace([np.inf, -np.inf], np.nan)
    movs = movs.dropna()
    movs["climb"] = (
        np.sign(movs["delta_alt_m"])
        .replace(0.0, 1.0)
        .map({1.0: "Uphill", -1.0: "Downhill"})
        .astype("category")
    )
    fig, ax = plt.subplots()
    sns.kdeplot(x="speed_kmh", fill=True, hue="climb", data=movs, cut=0, ax=ax)
    ax.grid(visible=False)
    ax.set_ylabel("")
    ax.set_yticks([])
    ax.get_legend().set_title("TERRAIN")
    return fig



"""Visualize Tracking Data."""
from .__main__ import main
from .heatmap import heatmap
from .heatmap import heatmap_from_dataframe
from .input_file import gpx_to_dataframe
from .input_file import tcx_to_dataframe
from .input_file import TrackingColumn
from .speed import plot_acceleration
from .speed import plot_movement_field
from .speed import plot_speed
from .speed import plot_speed_moving_avg
from .speed import track_2_movements
from .speed import web_plot_speed_climb_kde
from .speed import web_plot_speed_elevation
from .webserver import run_webserver

__all__ = [
    "main",
    "heatmap",
    "heatmap_from_dataframe",
    "gpx_to_dataframe",
    "tcx_to_dataframe",
    "TrackingColumn",
    "plot_acceleration",
    "plot_movement_field",
    "plot_speed",
    "plot_speed_moving_avg",
    "track_2_movements",
    "web_plot_speed_climb_kde",
    "web_plot_speed_elevation",
    "run_webserver",
]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['track_viz']

package_data = \
{'': ['*'], 'track_viz': ['static/*', 'templates/*']}

install_requires = \
['Flask>=2.0.2,<3.0.0',
 'PyYAML>=6.0,<7.0',
 'click>=8.0.1,<9.0.0',
 'defusedxml>=0.7.1,<0.8.0',
 'geopy>=2.2.0,<3.0.0',
 'lxml>=4.6.4,<5.0.0',
 'matplotlib>=3.5.0,<4.0.0',
 'pandas>=1.3.4,<2.0.0',
 'poetry>=1.1.12,<2.0.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'seaborn>=0.11.2,<0.12.0']

entry_points = \
{'console_scripts': ['track-viz = track_viz.__main__:main']}

setup_kwargs = {
    'name': 'track-viz',
    'version': '0.1.0',
    'description': 'Visualize Tracking Data',
    'long_description': "Track Visualization\n===================\n\n|PyPI| |Status| |Python Version| |License|\n\n|Read the Docs| |Tests| |Codecov|\n\n|pre-commit| |Black|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/track-viz.svg\n   :target: https://pypi.org/project/track-viz/\n   :alt: PyPI\n.. |Status| image:: https://img.shields.io/pypi/status/track-viz.svg\n   :target: https://pypi.org/project/track-viz/\n   :alt: Status\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/track-viz\n   :target: https://pypi.org/project/track-viz\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/track-viz\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/track-viz/latest.svg?label=Read%20the%20Docs\n   :target: https://track-viz.readthedocs.io/\n   :alt: Read the documentation at https://track-viz.readthedocs.io/\n.. |Tests| image:: https://github.com/JulienMBABD/track-viz/workflows/Tests/badge.svg\n   :target: https://github.com/JulienMBABD/track-viz/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://codecov.io/gh/JulienMBABD/track-viz/branch/main/graph/badge.svg\n   :target: https://codecov.io/gh/JulienMBABD/track-viz\n   :alt: Codecov\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\n\nFeatures\n--------\n\n* TODO\n\n\nRequirements\n------------\n\n* TODO\n\n\nInstallation\n------------\n\nYou can install *Football Track* via pip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install track-viz\n\n\nUsage\n-----\n\nPlease see the `Command-line Reference <Usage_>`_ for details.\n\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\nDistributed under the terms of the `MIT license`_,\n*Football Track* is free and open source software.\n\n\nIssues\n------\n\nIf you encounter any problems,\nplease `file an issue`_ along with a detailed description.\n\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_'s `Hypermodern Python Cookiecutter`_ template.\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _MIT license: https://opensource.org/licenses/MIT\n.. _PyPI: https://pypi.org/\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _file an issue: https://github.com/JulienMBABD/track-viz/issues\n.. _pip: https://pip.pypa.io/\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n.. _Usage: https://track-viz.readthedocs.io/en/latest/usage.html\n",
    'author': 'Julien Rossi',
    'author_email': 'mr.julien.rossi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/JulienMBABD/track-viz',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Run a Flask web server to create heatmap and speed graph."""
from base64 import b64encode
from io import BytesIO
from io import StringIO
from pathlib import Path
from secrets import token_hex
from tempfile import gettempdir

import flask.typing as ft
from flask import Flask
from flask import Markup  # type: ignore
from flask import redirect
from flask import render_template
from flask import request
from werkzeug.utils import secure_filename

from .heatmap import heatmap_from_dataframe
from .input_file import gpx_to_dataframe
from .input_file import tcx_to_dataframe
from .speed import web_plot_speed_climb_kde
from .speed import web_plot_speed_elevation


app = Flask(__name__)
ALLOWED_EXTENSIONS = {".tcx", ".gpx"}
app.config["MAX_CONTENT_LENGTH"] = 4 * 1024 * 1024  # 4MB
app.config["UPLOAD_FOLDER"] = gettempdir()


def _allowed_file(filename: str) -> bool:
    return Path(filename).suffix in ALLOWED_EXTENSIONS


@app.route("/heatmap", methods=["GET", "POST"])
def create_heatmap() -> ft.ResponseReturnValue:
    """Handles incoming activity file and create heatmap."""
    if request.method == "POST":
        f = request.files["file"]

        if f.filename is None or not _allowed_file(f.filename):
            return redirect(request.url)

        fpath = (
            Path(app.config["UPLOAD_FOLDER"])
            / f"{token_hex(8)}_{secure_filename(f.filename)}"
        )
        f.save(fpath)

        suffix = fpath.suffix
        if suffix == ".tcx":
            track = tcx_to_dataframe(tcx=fpath)
        elif suffix == ".gpx":
            track = gpx_to_dataframe(gpx=fpath)
        else:
            raise ValueError(
                f"Wrong suffix {suffix}, expected one of {app.config['ALLOWED_EXTENSIONS']}"
            )

        fig = heatmap_from_dataframe(track=track, config=Path("static/heatmap.yml"))
        img_bytes = BytesIO()
        fig.savefig(img_bytes, format="jpg")
        img_b64bytes = b64encode(img_bytes.getvalue()).decode("utf-8")

        fpath.unlink()

        return render_template("show_heatmap.html", img_data=img_b64bytes)
    else:
        return render_template("upload_heatmap.html")


@app.route("/speed", methods=["GET", "POST"])
def create_speed_plots() -> ft.ResponseReturnValue:
    """Handles incoming activity file and create speed graph."""
    if request.method == "POST":
        f = request.files["file"]

        if f.filename is None or not _allowed_file(f.filename):
            return redirect(request.url)

        fpath = (
            Path(app.config["UPLOAD_FOLDER"])
            / f"{token_hex(8)}_{secure_filename(f.filename)}"
        )
        f.save(fpath)

        suffix = fpath.suffix
        if suffix == ".tcx":
            track = tcx_to_dataframe(tcx=fpath)
        elif suffix == ".gpx":
            track = gpx_to_dataframe(gpx=fpath)
        else:
            raise ValueError(
                f"Wrong suffix {suffix}, expected one of {app.config['ALLOWED_EXTENSIONS']}"
            )

        fig = web_plot_speed_elevation(track=track)
        speed_terrain_xml = StringIO()
        fig.savefig(speed_terrain_xml, format="svg")

        fig = web_plot_speed_climb_kde(track=track)
        speed_terrain_kde_xml = StringIO()
        fig.savefig(speed_terrain_kde_xml, format="svg")

        fpath.unlink()

        return render_template(
            "show_graph.html",
            page_title="Speed Graphs",
            img_speed_terrain=Markup(speed_terrain_xml.getvalue()),
            img_speed_terrain_kde=Markup(speed_terrain_kde_xml.getvalue()),
        )
    else:
        return render_template("upload_speed.html")


def run_webserver(host: str, port: int) -> None:
    """Run webserver."""
    app.run(host=host, port=port, debug=False)
